﻿Cartographer = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceModuleCore-2.0")
Cartographer.version = "2.0." .. string.sub("$Revision: 17389 $", 12, -3)
Cartographer.date = string.sub("$Date: 2006-11-18 13:28:36 -0500 (Sat, 18 Nov 2006) $", 8, 17)

local Dewdrop = AceLibrary("Dewdrop-2.0")

local L = AceLibrary("AceLocale-2.2"):new("Cartographer")

L:RegisterTranslations("enUS", function() return {
	["Toggle"] = true,
	["Toggle the module on and off."] = true,
	
	["Right-Click on map to zoom out"] = true,
	["Left-Click on map to zoom in"] = true,
	
	["Go to"] = true,
} end)

L:RegisterTranslations("deDE", function() return {
--	["Toggle"] = true,
--	["Toggle the module on and off."] = true,
} end)

L:RegisterTranslations("frFR", function() return {
--	["Toggle"] = true,
--	["Toggle the module on and off."] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Toggle"] = "토글",
	["Toggle the module on and off."] = "모듈을 켜거나 끔니다",
	
	["Right-Click on map to zoom out"] = "오른쪽-클릭 : 축소",
	["Left-Click on map to zoom in"] = "왼쪽-클릭 : 확대",
	
	["Go to"] = "바로가기",
} end)

L:RegisterTranslations("zhCN", function() return {
--	["Toggle"] = true,
--	["Toggle the module on and off."] = true,
} end)

L:RegisterTranslations("zhTW", function() return {
--	["Toggle"] = true,
--	["Toggle the module on and off."] = true,
} end)

Cartographer:RegisterDB("CartographerDB")

Cartographer.options = {
	type = 'group',
	args = {
	},
	handler = Cartographer
}
Dewdrop:InjectAceOptionsTable(Cartographer, Cartographer.options)
Cartographer.options.args.standby = nil

function Cartographer:OnInitialize()
	self.options.args.goto = {
		name = L["Go to"],
		desc = L["Go to"],
		type = 'group',
		args = {
		},
		order = 1,
	}
	self.options.args.goto_space = {
		type = 'header',
		order = 1.1,
	}
	
	for i,continent in ipairs { GetMapContinents() } do
		local i = i
		local validate = { GetMapZones(i) }
		self.options.args.goto.args[string.gsub(continent, " ", "-")] = {
			name = continent,
			desc = continent,
			type = 'text',
			validate = validate,
			get = function() return self:GetCurrentInstance() or GetCurrentMapContinent() == i and validate[GetCurrentMapZone()] end,
			set = function(text)
				for j,v in ipairs(validate) do
					if v == text then
						SetMapZoom(i, j)
						break
					end
				end
			end,
		}
	end
end

local magnifyingGlassTexts = { L["Right-Click on map to zoom out"], L["Left-Click on map to zoom in"] }
local oldMagnifyingGlassText
function Cartographer:OnEnable()
	local button = CreateFrame("Button", "CartographerOptionsButton", WorldMapFrame, "UIPanelButtonTemplate")
	button:SetText("Cartographer")
	local width = button:GetTextWidth() + 30
	if width < 110 then
		width = 110
	end
	button:SetWidth(width)
	button:SetHeight(22)
	WorldMapZoomOutButton:Hide()
	button:SetPoint("LEFT", WorldMapZoomOutButton, "LEFT", 0, 0)
	button:SetScript("OnClick", function()
		Dewdrop:Register(this,
			'children', self.options,
			'dontHook', true,
			'point', "TOPRIGHT",
			'relativePoint', "BOTTOMRIGHT"
		)
		this:SetScript("OnClick", function()
			if Dewdrop:IsOpen(this) then
				Dewdrop:Close()
			else
				Dewdrop:Open(this)
			end
		end)
		this:GetScript("OnClick")()
	end)
	oldMagnifyingGlassText = WorldMapMagnifyingGlassButton:GetText()
	WorldMapMagnifyingGlassButton:SetText(table.concat(magnifyingGlassTexts, "\n"))
end

function Cartographer:OnDisable()
	WorldMapMagnifyingGlassButton:SetText(oldMagnifyingGlassText)
end

function Cartographer:OnProfileEnable(alpha, bravo)
	for _,module in self:IterateModules() do
		if self:IsModuleActive(module) and type(module.OnProfileEnable) == "function" then
			module:OnProfileEnable(alpha, bravo)
		end
	end
end

function Cartographer:AddToMagnifyingGlass(text)
	for _,v in ipairs(magnifyingGlassTexts) do
		if v == text then
			error(string.format("Cannot add %q to magnifying glass, it already exists", text), 2)
		end
	end
	table.insert(magnifyingGlassTexts, text)
	WorldMapMagnifyingGlassButton:SetText(table.concat(magnifyingGlassTexts, "\n"))
end

function Cartographer:RemoveFromMagnifyingGlass(text)
	local id
	for i,v in ipairs(magnifyingGlassTexts) do
		if v == text then
			id = i
			break
		end
	end
	if not id then
		error(string.format("Cannot remove %q from magnifying glass, it does not exist", text), 2)
	end
	table.remove(magnifyingGlassTexts, id)
	
	WorldMapMagnifyingGlassButton:SetText(table.concat(magnifyingGlassTexts, "\n"))
end

local currentInstance
function Cartographer:SetCurrentInstance(zone)
	currentInstance = zone
	self:TriggerEvent("Cartographer_SetCurrentInstance", zone)
end

function Cartographer:GetCurrentInstance()
	return currentInstance
end

local instanceWorldMapButton
function Cartographer:RegisterInstanceWorldMapButton(frame)
	instanceWorldMapButton = frame
	self:TriggerEvent("Cartographer_RegisterInstanceWorldMapButton", frame)
end

function Cartographer:GetInstanceWorldMapButton()
	return instanceWorldMapButton
end
